/*
 * Copyright 2014-2015 ISP RAS (http://www.ispras.ru)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.test.sequence.engine.allocator;

import java.util.Map;
import java.util.Set;

/**
 * {@link AllocationStrategy} defines an interface of resource allocation strategies.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public interface AllocationStrategy {

  /**
   * Chooses an object.
   * 
   * @param <T> type of objects.
   * @param free the set of free objects.
   * @param used the set of used objects.
   * @param attributes the parameters.
   * @return the chosen object or {@code null}.
   */
  <T> T next(final Set<T> free, final Set<T> used, final Map<String, String> attributes);
}
